@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -66,7 +66,7 @@ class CControllerPopupMediatypeTestSend extends CController {
 	protected function validateMediaType() {
 		$mediatypes = API::MediaType()->get([
 			'output' => ['type', 'status'],
-			'mediatypeids' => $this->getInput('mediatypeid'),
+			'mediatypeids' => $this->getInput('mediatypeid')
 		]);
 
 		if (!$mediatypes) {
