@@ -85,16 +85,20 @@ else {
 	foreach ($data['images'] as $image) {
 		$img = ($image['imagetype'] == IMAGE_TYPE_BACKGROUND)
 			? new CLink(
-				new CImg('imgstore.php?width=200&height=200&iconid='.$image['imageid'], 'no image'),
+				(new CImg('#', 'no image'))
+					->setAttribute('data-src', 'imgstore.php?width=200&height=200&iconid='.$image['imageid'])
+					->addStyle('display: none;'),
 				'image.php?imageid='.$image['imageid']
 			)
-			: new CImg('imgstore.php?iconid='.$image['imageid'], 'no image');
+			: (new CImg('#', 'no image'))->setAttribute('data-src', 'imgstore.php?iconid='.$image['imageid'])
+				->addStyle('display: none;');
 
 		$edit_url->setArgument('imageid', $image['imageid']);
 
 		$image_row->addItem(
 			(new CDiv())
 				->addClass(ZBX_STYLE_CELL)
+				->addClass('lazyload-image')
 				->addItem([$img, BR(), new CLink($image['name'], $edit_url->getUrl())])
 		);
 
@@ -116,3 +120,11 @@ else {
 }
 
 $widget->show();
+
+(new CScriptTag('
+	view.init('.json_encode([
+		'load_images' => count($data['images'])
+	]).');
+'))
+	->setOnDocumentReady()
+	->show();
