@@ -1,4 +1,4 @@
-<?php declare(strict_types=1);
+<?php declare(strict_types = 0);
 /*
 ** Zabbix
 ** Copyright (C) 2001-2022 Zabbix SIA
@@ -62,7 +62,7 @@ $token_from_list = (new CFormList())
 	)
 	->addRow((new CLabel(_('Expires at')))->setAsteriskMark(),
 		(new CDateSelector('expires_at', $data['expires_at']))
-			->setDateFormat(DATE_TIME_FORMAT_SECONDS)
+			->setDateFormat(ZBX_FULL_DATE_TIME)
 			->setPlaceholder(_('YYYY-MM-DD hh:mm:ss'))
 			->setAriaRequired(),
 		'expires-at-row'
