@@ -1,4 +1,4 @@
-<?php declare(strict_types=1);
+<?php declare(strict_types = 0);
 /*
 ** Zabbix
 ** Copyright (C) 2001-2022 Zabbix SIA
@@ -29,8 +29,8 @@ if ($data['uncheck']) {
 
 $this->includeJsFile('administration.user.token.list.js.php');
 
-$filter = (new CFilter((new CUrl('zabbix.php'))->setArgument('action', 'user.token.list')));
-$filter
+$filter = (new CFilter())
+	->setResetUrl((new CUrl('zabbix.php'))->setArgument('action', 'user.token.list'))
 	->addVar('action', 'user.token.list')
 	->setProfile($data['profileIdx'])
 	->setActiveTab($data['active_tab'])
