@@ -337,7 +337,7 @@ class CGraphPrototype extends CGraphGeneral {
 
 		CGraphPrototypeManager::delete($graphids);
 
-		$this->addAuditBulk(AUDIT_ACTION_DELETE, AUDIT_RESOURCE_GRAPH_PROTOTYPE, $db_graphs);
+		$this->addAuditBulk(CAudit::ACTION_DELETE, CAudit::RESOURCE_GRAPH_PROTOTYPE, $db_graphs);
 
 		return ['graphids' => $graphids];
 	}
