@@ -158,11 +158,13 @@ abstract class CHostGeneral extends CHostBase {
 	 * @param array  $templateids
 	 * @param array  $db_hosts
 	 */
-	protected function massCheckTemplatesLinks(string $method, array $templateids, array $db_hosts): void {
+	protected function massCheckTemplatesLinks(string $method, array $templateids, array $db_hosts,
+			array $templateids_clear = []): void {
 		$ins_templates = [];
 		$del_links = [];
 		$check_double_linkage = false;
 		$del_templates = [];
+		$del_links_clear  = [];
 
 		foreach ($db_hosts as $hostid => $db_host) {
 			$db_templateids = array_column($db_host['templates'], 'templateid');
@@ -211,14 +213,21 @@ abstract class CHostGeneral extends CHostBase {
 				if ($upd_templateids) {
 					$del_templates[$db_templateid][$hostid] = $upd_templateids;
 				}
+
+				if (array_key_exists($db_templateid, $templateids_clear)) {
+					$del_links_clear[$db_templateid][$hostid] = true;
+				}
 			}
 		}
 
 		if ($del_templates) {
-			$this->checkTriggerDependenciesOfUpdTemplates($del_templates);
 			$this->checkTriggerExpressionsOfDelTemplates($del_templates);
 		}
 
+		if ($del_links_clear) {
+			$this->checkTriggerDependenciesOfHostTriggers($del_links_clear);
+		}
+
 		if ($ins_templates) {
 			if ($this instanceof CTemplate) {
 				self::checkCircularLinkageNew($ins_templates, $del_links);
@@ -760,11 +769,7 @@ abstract class CHostGeneral extends CHostBase {
 
 		API::Graph()->syncTemplates($link_request);
 
-		API::Trigger()->syncTemplateDependencies($link_request);
-
-		if ($ruleids) {
-			API::TriggerPrototype()->syncTemplateDependencies($link_request);
-		}
+		CTriggerGeneral::syncTemplateDependencies($link_request['templateids'], $link_request['hostids']);
 	}
 
 	/**
@@ -998,8 +1003,7 @@ abstract class CHostGeneral extends CHostBase {
 		}
 
 		foreach ($link_requests as $link_request){
-			API::Trigger()->syncTemplateDependencies($link_request);
-			API::TriggerPrototype()->syncTemplateDependencies($link_request);
+			CTriggerGeneral::syncTemplateDependencies($link_request['templateids'], $link_request['hostids']);
 		}
 
 		return $hosts_linkage_inserts;
