@@ -1,7 +1,7 @@
-<?php declare(strict_types = 1);
+<?php declare(strict_types = 0);
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -29,14 +29,7 @@ class CSettings extends CApiService {
 		'update' => ['min_user_type' => USER_TYPE_SUPER_ADMIN]
 	];
 
-	/**
-	 * @var string
-	 */
 	protected $tableName = 'config';
-
-	/**
-	 * @var string
-	 */
 	protected $tableAlias = 'c';
 
 	/**
@@ -52,12 +45,11 @@ class CSettings extends CApiService {
 		'snmptrap_logging', 'default_lang', 'default_timezone', 'login_attempts', 'login_block', 'validate_uri_schemes',
 		'uri_valid_schemes', 'x_frame_options', 'iframe_sandboxing_enabled', 'iframe_sandboxing_exceptions',
 		'max_overview_table_size', 'connect_timeout', 'socket_timeout', 'media_type_test_timeout', 'script_timeout',
-		'item_test_timeout', 'url', 'report_test_timeout'
+		'item_test_timeout', 'url', 'report_test_timeout', 'auditlog_enabled', 'ha_failover_delay',
+		'geomaps_tile_provider', 'geomaps_tile_url', 'geomaps_max_zoom', 'geomaps_attribution'
 	];
 
 	/**
-	 * Get settings parameters.
-	 *
 	 * @param array $options
 	 *
 	 * @throws APIException if the input is invalid.
@@ -68,6 +60,7 @@ class CSettings extends CApiService {
 		$api_input_rules = ['type' => API_OBJECT, 'fields' => [
 			'output' =>	['type' => API_OUTPUT, 'in' => implode(',', $this->output_fields), 'default' => API_OUTPUT_EXTEND]
 		]];
+
 		if (!CApiInputValidator::validate($api_input_rules, $options, '/', $error)) {
 			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 		}
@@ -88,17 +81,16 @@ class CSettings extends CApiService {
 	}
 
 	/**
-	 * Get global settings parameters.
-	 *
 	 * @param array $options
+	 * @param bool  $api_call  Flag indicating whether this method called via an API call or from a local PHP file.
 	 *
 	 * @throws APIException if the input is invalid.
 	 *
 	 * @return array
 	 */
-	public function getGlobal(array $options, $api_call = true): array {
+	public function getGlobal(array $options, bool $api_call = true): array {
 		if ($api_call) {
-			return self::exception(ZBX_API_ERROR_PARAMETERS,
+			self::exception(ZBX_API_ERROR_PARAMETERS,
 				_s('Incorrect method "%1$s.%2$s".', 'settings', 'getglobal')
 			);
 		}
@@ -106,11 +98,12 @@ class CSettings extends CApiService {
 		$output_fields = ['default_theme', 'show_technical_errors', 'severity_color_0', 'severity_color_1',
 			'severity_color_2', 'severity_color_3', 'severity_color_4', 'severity_color_5', 'custom_color',
 			'problem_unack_color', 'problem_ack_color', 'ok_unack_color', 'ok_ack_color', 'default_lang',
-			'x_frame_options', 'default_timezone', 'session_key'
+			'x_frame_options', 'default_timezone', 'session_key', 'dbversion_status'
 		];
 		$api_input_rules = ['type' => API_OBJECT, 'fields' => [
 			'output' =>	['type' => API_OUTPUT, 'in' => implode(',', $output_fields), 'default' => API_OUTPUT_EXTEND]
 		]];
+
 		if (!CApiInputValidator::validate($api_input_rules, $options, '/', $error)) {
 			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 		}
@@ -131,44 +124,22 @@ class CSettings extends CApiService {
 	}
 
 	/**
-	 * Update settings parameters.
-	 *
 	 * @param array $settings
 	 *
+	 * @throws APIException if the input is invalid.
+	 *
 	 * @return array
 	 */
 	public function update(array $settings): array {
+		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
+			self::exception(ZBX_API_ERROR_PERMISSIONS,
+				_s('No permissions to call "%1$s.%2$s".', 'settings', __FUNCTION__)
+			);
+		}
+
 		$db_settings = $this->validateUpdate($settings);
 
-		$upd_config = [];
-
-		// strings
-		$field_names = ['default_theme', 'work_period', 'history_period', 'period_default', 'max_period',
-			'severity_color_0', 'severity_color_1', 'severity_color_2', 'severity_color_3', 'severity_color_4',
-			'severity_color_5', 'severity_name_0', 'severity_name_1', 'severity_name_2', 'severity_name_3',
-			'severity_name_4', 'severity_name_5', 'ok_period', 'blink_period', 'problem_unack_color',
-			'problem_ack_color', 'ok_unack_color', 'ok_ack_color', 'default_lang',
-			'default_timezone', 'login_block', 'uri_valid_schemes', 'x_frame_options', 'iframe_sandboxing_exceptions',
-			'connect_timeout', 'socket_timeout', 'media_type_test_timeout', 'script_timeout', 'item_test_timeout',
-			'url', 'report_test_timeout'
-		];
-		foreach ($field_names as $field_name) {
-			if (array_key_exists($field_name, $settings) && $settings[$field_name] !== $db_settings[$field_name]) {
-				$upd_config[$field_name] = $settings[$field_name];
-			}
-		}
-
-		// integers
-		$field_names = ['search_limit', 'max_in_table', 'server_check_interval', 'show_technical_errors',
-			'custom_color', 'problem_unack_style', 'problem_ack_style', 'ok_unack_style', 'ok_ack_style',
-			'discovery_groupid', 'default_inventory_mode', 'alert_usrgrpid', 'snmptrap_logging', 'login_attempts',
-			'validate_uri_schemes', 'iframe_sandboxing_enabled', 'max_overview_table_size'
-		];
-		foreach ($field_names as $field_name) {
-			if (array_key_exists($field_name, $settings) && $settings[$field_name] != $db_settings[$field_name]) {
-				$upd_config[$field_name] = $settings[$field_name];
-			}
-		}
+		$upd_config = DB::getUpdatedValues('config', $settings, $db_settings);
 
 		if ($upd_config) {
 			DB::update('config', [
@@ -177,13 +148,13 @@ class CSettings extends CApiService {
 			]);
 
 			if (array_key_exists('discovery_groupid', $upd_config)
-					&& bccomp($upd_config['discovery_groupid'], $db_settings['discovery_groupid']) !== 0) {
+					&& bccomp($upd_config['discovery_groupid'], $db_settings['discovery_groupid']) != 0) {
 				$this->setHostGroupInternal($db_settings['discovery_groupid'], ZBX_NOT_INTERNAL_GROUP);
 				$this->setHostGroupInternal($upd_config['discovery_groupid'], ZBX_INTERNAL_GROUP);
 			}
 		}
 
-		$this->addAuditBulk(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_SETTINGS,
+		self::addAuditLog(CAudit::ACTION_UPDATE, CAudit::RESOURCE_SETTINGS,
 			[['configid' => $db_settings['configid']] + $settings], [$db_settings['configid'] => $db_settings]
 		);
 
@@ -191,15 +162,13 @@ class CSettings extends CApiService {
 	}
 
 	/**
-	 * Validate updated settings parameters.
-	 *
-	 * @param array  $settings
+	 * @param array $settings
 	 *
 	 * @throws APIException if the input is invalid.
 	 *
 	 * @return array
 	 */
-	protected function validateUpdate(array $settings): array {
+	protected function validateUpdate(array &$settings): array {
 		$api_input_rules = ['type' => API_OBJECT, 'flags' => API_NOT_EMPTY, 'fields' => [
 			'default_theme' =>					['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'in' => implode(',', array_keys(APP::getThemes()))],
 			'search_limit' =>					['type' => API_INT32, 'in' => '1:999999'],
@@ -238,7 +207,7 @@ class CSettings extends CApiService {
 			'alert_usrgrpid' =>					['type' => API_ID, 'flags' => API_ALLOW_NULL],
 			'snmptrap_logging' =>				['type' => API_INT32, 'in' => '0,1'],
 			'default_lang' =>					['type' => API_STRING_UTF8, 'in' => implode(',', array_keys(getLocales()))],
-			'default_timezone' =>				['type' => API_STRING_UTF8, 'in' => ZBX_DEFAULT_TIMEZONE.','.implode(',', array_keys((new CDateTimeZoneHelper())->getAllDateTimeZones()))],
+			'default_timezone' =>				['type' => API_STRING_UTF8, 'in' => ZBX_DEFAULT_TIMEZONE.','.implode(',', array_keys(CTimezoneHelper::getList()))],
 			'login_attempts' =>					['type' => API_INT32, 'in' => '1:32'],
 			'login_block' =>					['type' => API_TIME_UNIT, 'flags' => API_NOT_EMPTY, 'in' => implode(':', [30, SEC_PER_HOUR])],
 			'validate_uri_schemes' =>			['type' => API_INT32, 'in' => '0,1'],
@@ -253,17 +222,18 @@ class CSettings extends CApiService {
 			'script_timeout' =>					['type' => API_TIME_UNIT, 'flags' => API_NOT_EMPTY, 'in' => '1:300'],
 			'item_test_timeout' =>				['type' => API_TIME_UNIT, 'flags' => API_NOT_EMPTY, 'in' => '1:300'],
 			'url' =>							['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('config', 'url')],
-			'report_test_timeout' =>			['type' => API_TIME_UNIT, 'flags' => API_NOT_EMPTY, 'in' => '1:300']
+			'report_test_timeout' =>			['type' => API_TIME_UNIT, 'flags' => API_NOT_EMPTY, 'in' => '1:300'],
+			'auditlog_enabled' =>				['type' => API_INT32, 'in' => '0,1'],
+			'geomaps_tile_provider' =>			['type' => API_STRING_UTF8, 'in' => ','.implode(',', array_keys(getTileProviders()))],
+			'geomaps_tile_url' =>				['type' => API_URL, 'length' => DB::getFieldLength('config', 'geomaps_tile_url')],
+			'geomaps_max_zoom' =>				['type' => API_INT32, 'in' => '0:'.ZBX_GEOMAP_MAX_ZOOM],
+			'geomaps_attribution' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('config', 'geomaps_attribution')]
 		]];
+
 		if (!CApiInputValidator::validate($api_input_rules, $settings, '/', $error)) {
 			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 		}
 
-		// Check permissions.
-		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
-		}
-
 		if (array_key_exists('discovery_groupid', $settings)) {
 			$db_hstgrp_exists = API::HostGroup()->get([
 				'countOutput' => true,
@@ -271,6 +241,7 @@ class CSettings extends CApiService {
 				'filter' => ['flags' => ZBX_FLAG_DISCOVERY_NORMAL],
 				'editable' => true
 			]);
+
 			if (!$db_hstgrp_exists) {
 				self::exception(ZBX_API_ERROR_PARAMETERS,
 					_s('Host group with ID "%1$s" is not available.', $settings['discovery_groupid'])
@@ -283,6 +254,7 @@ class CSettings extends CApiService {
 				'countOutput' => true,
 				'usrgrpids' => $settings['alert_usrgrpid']
 			]);
+
 			if (!$db_usrgrp_exists) {
 				self::exception(ZBX_API_ERROR_PARAMETERS,
 					_s('User group with ID "%1$s" is not available.', $settings['alert_usrgrpid'])
@@ -290,6 +262,12 @@ class CSettings extends CApiService {
 			}
 		}
 
+		if (array_key_exists('geomaps_tile_provider', $settings) && $settings['geomaps_tile_provider'] !== '') {
+			$settings['geomaps_tile_url'] = DB::getDefault('config', 'geomaps_tile_url');
+			$settings['geomaps_max_zoom'] = DB::getDefault('config', 'geomaps_max_zoom');
+			$settings['geomaps_attribution'] = DB::getDefault('config', 'geomaps_attribution');
+		}
+
 		$period_default_updated = array_key_exists('period_default', $settings);
 		$max_period_updated = array_key_exists('max_period', $settings);
 		if ($period_default_updated || $max_period_updated) {
@@ -304,6 +282,7 @@ class CSettings extends CApiService {
 			if ($period_default > $max_period) {
 				$field = 'period_default';
 				$message = _('time filter default period exceeds the max period');
+
 				if (!$period_default_updated) {
 					$field = 'max_period';
 					$message = _('max period is less than time filter default period');
@@ -327,7 +306,7 @@ class CSettings extends CApiService {
 	 * @param string $groupid   Host group ID
 	 * @param int    $internal  Value of internal option
 	 */
-	private function setHostGroupInternal($groupid, $internal): void {
+	private function setHostGroupInternal(string $groupid, int $internal): void {
 		DB::update('hstgrp', [
 			'values' => ['internal' =>  $internal],
 			'where' => ['groupid' => $groupid]
