@@ -45,6 +45,29 @@ class CWidgetHelper {
 	 * @return CFormList
 	 */
 	public static function createFormList($name, $type, $view_mode, $known_widget_types, $field_rf_rate) {
+		$deprecated_types = array_intersect_key(
+			$known_widget_types,
+			array_flip(CWidgetConfig::DEPRECATED_WIDGETS)
+		);
+		$known_widget_types = array_diff_key($known_widget_types, $deprecated_types);
+		$types_select = (new CSelect('type'))
+			->setFocusableElementId('label-type')
+			->setId('type')
+			->setValue($type)
+			->setAttribute('autofocus', 'autofocus')
+			->addOptions(CSelect::createOptionsFromArray($known_widget_types));
+
+		if ($deprecated_types) {
+			$types_select->addOptionGroup(
+				(new CSelectOptionGroup(_('Deprecated')))->addOptions(
+					CSelect::createOptionsFromArray($deprecated_types)
+			));
+		}
+
+		if (array_key_exists($type, $deprecated_types)) {
+			$types_select = [$types_select, ' ', makeWarningIcon(_('Widget is deprecated.'))];
+		}
+
 		$form_list = (new CFormList())
 			->addItem((new CListItem([
 					(new CDiv(new CLabel(_('Type'), 'label-type')))->addClass(ZBX_STYLE_TABLE_FORMS_TD_LEFT),
@@ -55,12 +78,7 @@ class CWidgetHelper {
 							->setId('show_header')
 							->setChecked($view_mode == ZBX_WIDGET_VIEW_MODE_NORMAL)
 						))->addClass(ZBX_STYLE_TABLE_FORMS_SECOND_COLUMN),
-						(new CSelect('type'))
-							->setFocusableElementId('label-type')
-							->setId('type')
-							->setValue($type)
-							->setAttribute('autofocus', 'autofocus')
-							->addOptions(CSelect::createOptionsFromArray($known_widget_types))
+						$types_select
 					]))->addClass(ZBX_STYLE_TABLE_FORMS_TD_RIGHT)
 				]))->addClass('table-forms-row-with-second-field')
 			)
@@ -111,17 +129,25 @@ class CWidgetHelper {
 	 * Creates label linked to the field.
 	 *
 	 * @param CWidgetField $field
+	 * @param string       $class	Custom CSS class for label.
+	 * @param mixed        $hint	Hint box text.
 	 *
 	 * @return CLabel
 	 */
-	public static function getLabel($field) {
+	public static function getLabel($field, $class = null, $hint = null) {
 		if ($field instanceof CWidgetFieldSelect) {
 			return (new CLabel($field->getLabel(), 'label-'.$field->getName()))
-				->setAsteriskMark(self::isAriaRequired($field));
+				->setAsteriskMark(self::isAriaRequired($field))
+				->addClass($class);
 		}
 
-		return (new CLabel($field->getLabel(), $field->getName()))
-			->setAsteriskMark(self::isAriaRequired($field));
+		$help_icon = ($hint !== null)
+			? makeHelpIcon($hint)
+			: null;
+
+		return (new CLabel([$field->getLabel(), $help_icon], $field->getName()))
+			->setAsteriskMark(self::isAriaRequired($field))
+			->addClass($class);
 	}
 
 	/**
@@ -139,6 +165,18 @@ class CWidgetHelper {
 			->setAriaRequired(self::isAriaRequired($field));
 	}
 
+	/**
+	 * @param CWidgetFieldTextArea $field
+	 *
+	 * @return CTextBox
+	 */
+	public static function getTextArea($field) {
+		return (new CTextArea($field->getName(), $field->getValue()))
+			->setAriaRequired(self::isAriaRequired($field))
+			->setEnabled(!($field->getFlags() & CWidgetField::FLAG_DISABLED))
+			->setAdaptiveWidth($field->getWidth());
+	}
+
 	/**
 	 * @param CWidgetFieldTextBox $field
 	 *
@@ -152,6 +190,17 @@ class CWidgetHelper {
 			->setWidth($field->getWidth());
 	}
 
+	/**
+	 * @param CWidgetFieldLatLng $field
+	 *
+	 * @return CTextBox
+	 */
+	public static function getLatLngZoomBox($field) {
+		return (new CTextBox($field->getName(), $field->getValue()))
+			->setAttribute('placeholder', $field->getPlaceholder())
+			->setWidth($field->getWidth());
+	}
+
 	/**
 	 * @param CWidgetFieldUrl $field
 	 *
@@ -189,6 +238,7 @@ class CWidgetHelper {
 			'object_name' => 'hosts',
 			'data' => $field->getValue(),
 			'placeholder' => $field->getPlaceholder(),
+			'wildcard_allowed' => 1,
 			'popup' => [
 				'parameters' => [
 					'srctbl' => 'hosts',
@@ -218,6 +268,21 @@ class CWidgetHelper {
 		];
 	}
 
+	/**
+	 * @param CWidgetFieldColor $field
+	 * @param bool              $use_default  Tell the Color picker whether to use Default color feature or not.
+	 *
+	 * @return CColor
+	 */
+	public static function getColor($field, $use_default = false) {
+		// appendColorPickerJs(false), because the script responsible for it is in widget.item.form.view.
+		$color_picker = (new CColor($field->getName(), $field->getValue()))->appendColorPickerJs(false);
+		if ($use_default) {
+			$color_picker->enableUseDefault();
+		}
+		return $color_picker;
+	}
+
 	/**
 	 * Creates label linked to the multiselect field.
 	 *
@@ -362,6 +427,40 @@ class CWidgetHelper {
 		] + $field->getFilterParameters());
 	}
 
+	/**
+	 * @param CWidgetFieldMsService $field
+	 * @param array $captions
+	 * @param string $form_name
+	 *
+	 * @return CMultiSelect
+	 */
+	public static function getService($field, $captions, $form_name) {
+		return (new CMultiSelect([
+			'name' => $field->getName().($field->isMultiple() ? '[]' : ''),
+			'object_name' => 'services',
+			'multiple' => $field->isMultiple(),
+			'data' => $captions,
+			'custom_select' => true,
+			'add_post_js' => false
+		]))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired(self::isAriaRequired($field));
+	}
+
+	/**
+	 * @param CWidgetFieldMsSla $field
+	 * @param array $captions
+	 * @param string $form_name
+	 *
+	 * @return CMultiSelect
+	 */
+	public static function getSla($field, $captions, $form_name) {
+		return self::getMultiselectField($field, $captions, $form_name, 'sla', [
+				'srctbl' => 'sla',
+				'srcfld1' => 'slaid'
+			] + $field->getFilterParameters());
+	}
+
 	public static function getSelectResource($field, $caption, $form_name) {
 		return [
 			(new CTextBox($field->getName().'_caption', $caption, true))
@@ -370,8 +469,9 @@ class CWidgetHelper {
 			(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 			(new CButton('select', _('Select')))
 				->addClass(ZBX_STYLE_BTN_GREY)
-				->onClick('return PopUp("popup.generic",'.
-					json_encode($field->getPopupOptions($form_name)).', null, this);')
+				->onClick('return PopUp("popup.generic", '.json_encode($field->getPopupOptions($form_name)).',
+					{dialogue_class: "modal-popup-generic"}
+				);')
 		];
 	}
 
@@ -395,8 +495,10 @@ class CWidgetHelper {
 	 *
 	 * @return CNumericBox
 	 */
-	public static function getIntegerBox($field) {
-		return (new CNumericBox($field->getName(), $field->getValue(), $field->getMaxLength()))
+	public static function getIntegerBox(CWidgetFieldIntegerBox $field): CNumericBox {
+		return (new CNumericBox($field->getName(), $field->getValue(), $field->getMaxLength(), false,
+			($field->getFlags() & CWidgetField::FLAG_NOT_EMPTY) == 0
+		))
 			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
 			->setAriaRequired(self::isAriaRequired($field));
 	}
@@ -447,12 +549,18 @@ class CWidgetHelper {
 
 	/**
 	 * @param CWidgetFieldCheckBoxList $field
-	 * @param array                    $list  Option list array.
+	 * @param array                    $list        Option list array.
+	 * @param array                    $class_list  List of additional CSS classes.
 	 *
 	 * @return CList
 	 */
-	public static function getCheckBoxList($field, array $list) {
+	public static function getCheckBoxList($field, array $list, array $class_list = []) {
 		$checkbox_list = (new CList())->addClass(ZBX_STYLE_LIST_CHECK_RADIO);
+		if ($class_list) {
+			foreach ($class_list as $class) {
+				$checkbox_list->addClass($class);
+			}
+		}
 
 		foreach ($list as $key => $label) {
 			$checkbox_list->addItem(
@@ -467,6 +575,69 @@ class CWidgetHelper {
 		return $checkbox_list;
 	}
 
+	/**
+	 * @param CWidgetFieldColumnsList $field  Widget columns field.
+	 *
+	 * @return CDiv
+	 */
+	public static function getWidgetColumns(CWidgetFieldColumnsList $field) {
+		$columns = $field->getValue();
+		$header = [
+			'',
+			(new CColHeader(_('Name')))->addStyle('width: 39%'),
+			(new CColHeader(_('Data')))->addStyle('width: 59%'),
+			_('Action')
+		];
+		$row_actions = [
+			(new CButton('edit', _('Edit')))
+				->addClass(ZBX_STYLE_BTN_LINK)
+				->removeId(),
+			(new CButton('remove', _('Remove')))
+				->addClass(ZBX_STYLE_BTN_LINK)
+				->removeId()
+		];
+		$table = (new CTable())
+			->setId('list_'.$field->getName())
+			->setHeader((new CRowHeader($header))->addClass($columns ? null : ZBX_STYLE_DISPLAY_NONE));
+		$enabled = !($field->getFlags() & CWidgetField::FLAG_DISABLED);
+
+		foreach ($columns as $column_index => $column) {
+			$column_data = [new CVar('sortorder['.$field->getName().'][]', $column_index)];
+
+			foreach ($column as $key => $value) {
+				$column_data[] = new CVar($field->getName().'['.$column_index.']['.$key.']', $value);
+			}
+
+			$label = array_key_exists('item', $column) ? $column['item'] : '';
+
+			if ($column['data'] == CWidgetFieldColumnsList::DATA_HOST_NAME) {
+				$label = new CTag('em', true, _('Host name'));
+			}
+			else if ($column['data'] == CWidgetFieldColumnsList::DATA_TEXT) {
+				$label = new CTag('em', true, $column['text']);
+			}
+
+			$table->addRow((new CRow([
+				(new CCol((new CDiv)->addClass(ZBX_STYLE_DRAG_ICON)))->addClass(ZBX_STYLE_TD_DRAG_ICON),
+				(new CDiv($column['name']))->addClass('text'),
+				(new CDiv($label))->addClass('text'),
+				(new CList(array_merge($row_actions, [$column_data])))->addClass(ZBX_STYLE_HOR_LIST)
+			]))->addClass('sortable'));
+		}
+
+		$table->addRow(
+			(new CCol(
+				(new CButton('add', _('Add')))
+					->addClass(ZBX_STYLE_BTN_LINK)
+					->setEnabled($enabled)
+			))->setColSpan(count($header))
+		);
+
+		return (new CDiv($table))
+			->addStyle('width: '.ZBX_TEXTAREA_STANDARD_WIDTH.'px')
+			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR);
+	}
+
 	/**
 	 * @param CWidgetFieldTags $field
 	 *
@@ -582,10 +753,11 @@ class CWidgetHelper {
 	 *
 	 * @return CDateSelector
 	 */
-	public static function getDatePicker($field) {
+	public static function getDatePicker(CWidgetFieldDatePicker $field): CDateSelector {
 		return (new CDateSelector($field->getName(), $field->getValue()))
 			->setAriaRequired(self::isAriaRequired($field))
-			->setEnabled(!($field->getFlags() & CWidgetField::FLAG_DISABLED));
+			->setMaxLength(DB::getFieldLength('widget_field', 'value_str'))
+			->setEnabled(($field->getFlags() & CWidgetField::FLAG_DISABLED) == 0);
 	}
 
 	/**
@@ -626,6 +798,7 @@ class CWidgetHelper {
 							'object_name' => 'hosts',
 							'data' => $value['hosts'],
 							'placeholder' => _('host pattern'),
+							'wildcard_allowed' => 1,
 							'popup' => [
 								'parameters' => [
 									'srctbl' => 'hosts',
@@ -647,6 +820,7 @@ class CWidgetHelper {
 							'data' => $value['items'],
 							'placeholder' => _('item pattern'),
 							'multiple' => true,
+							'wildcard_allowed' => 1,
 							'popup' => [
 								'parameters' => [
 									'srctbl' => 'items',
@@ -654,7 +828,6 @@ class CWidgetHelper {
 									'real_hosts' => 1,
 									'numeric' => 1,
 									'webitems' => 1,
-									'orig_names' => 1,
 									'dstfrm' => $form_name,
 									'dstfld1' => zbx_formatDomId($field->getName().'['.$row_num.'][items][]')
 								]
@@ -673,7 +846,7 @@ class CWidgetHelper {
 				(new CDiv(
 					(new CButton())
 						->setAttribute('title', _('Delete'))
-						->addClass(ZBX_STYLE_REMOVE_BTN)
+						->addClass(ZBX_STYLE_BTN_REMOVE)
 						->removeId()
 				))
 					->addClass(ZBX_STYLE_COLUMN_5)
@@ -781,7 +954,7 @@ class CWidgetHelper {
 				[
 					'name' => _('ADD OVERRIDE'),
 					'options' => [
-						['name' => _('Base colour'), 'callback' => 'addOverride', 'args' => ['color', '']],
+						['name' => _('Base color'), 'callback' => 'addOverride', 'args' => ['color', '']],
 
 						['name' => _('Width').'/0', 'callback' => 'addOverride', 'args' => ['width', 0]],
 						['name' => _('Width').'/1', 'callback' => 'addOverride', 'args' => ['width', 1]],
@@ -884,7 +1057,7 @@ class CWidgetHelper {
 				'.dynamicRows({'.
 					'template: "#overrides-row",'.
 					'beforeRow: ".overrides-foot",'.
-					'remove: ".'.ZBX_STYLE_REMOVE_BTN.'",'.
+					'remove: ".'.ZBX_STYLE_BTN_REMOVE.'",'.
 					'add: "#override-add",'.
 					'row: ".'.ZBX_STYLE_OVERRIDES_LIST_ITEM.'"'.
 				'})'.
@@ -986,6 +1159,7 @@ class CWidgetHelper {
 							'object_name' => 'hosts',
 							'data' => $value['hosts'],
 							'placeholder' => _('host pattern'),
+							'wildcard_allowed' => 1,
 							'popup' => [
 								'parameters' => [
 									'srctbl' => 'hosts',
@@ -1003,6 +1177,7 @@ class CWidgetHelper {
 							'object_name' => 'items',
 							'data' => $value['items'],
 							'placeholder' => _('item pattern'),
+							'wildcard_allowed' => 1,
 							'popup' => [
 								'parameters' => [
 									'srctbl' => 'items',
@@ -1010,7 +1185,6 @@ class CWidgetHelper {
 									'real_hosts' => 1,
 									'numeric' => 1,
 									'webitems' => 1,
-									'orig_names' => 1,
 									'dstfrm' => $form_name,
 									'dstfld1' => zbx_formatDomId($field_name.'['.$row_num.'][items][]')
 								]
@@ -1025,7 +1199,7 @@ class CWidgetHelper {
 				(new CDiv([
 					(new CButton())
 						->setAttribute('title', _('Delete'))
-						->addClass(ZBX_STYLE_REMOVE_BTN)
+						->addClass(ZBX_STYLE_BTN_REMOVE)
 						->removeId()
 				]))->addClass(ZBX_STYLE_COLUMN_5)
 			]))
@@ -1038,7 +1212,7 @@ class CWidgetHelper {
 					// Left column fields.
 					(new CDiv(
 						(new CFormList())
-							->addRow(_('Base colour'),
+							->addRow(_('Base color'),
 								(new CColor($field_name.'['.$row_num.'][color]', $value['color']))
 									->appendColorPickerJs(false)
 							)
@@ -1125,14 +1299,14 @@ class CWidgetHelper {
 									->setFocusableElementId('label-'.$field_name.'_'.$row_num.'_aggregate_function')
 									->setValue((int) $value['aggregate_function'])
 									->addOptions(CSelect::createOptionsFromArray([
-										GRAPH_AGGREGATE_NONE => graph_item_aggr_fnc2str(GRAPH_AGGREGATE_NONE),
-										GRAPH_AGGREGATE_MIN => graph_item_aggr_fnc2str(GRAPH_AGGREGATE_MIN),
-										GRAPH_AGGREGATE_MAX => graph_item_aggr_fnc2str(GRAPH_AGGREGATE_MAX),
-										GRAPH_AGGREGATE_AVG => graph_item_aggr_fnc2str(GRAPH_AGGREGATE_AVG),
-										GRAPH_AGGREGATE_COUNT => graph_item_aggr_fnc2str(GRAPH_AGGREGATE_COUNT),
-										GRAPH_AGGREGATE_SUM => graph_item_aggr_fnc2str(GRAPH_AGGREGATE_SUM),
-										GRAPH_AGGREGATE_FIRST => graph_item_aggr_fnc2str(GRAPH_AGGREGATE_FIRST),
-										GRAPH_AGGREGATE_LAST => graph_item_aggr_fnc2str(GRAPH_AGGREGATE_LAST)
+										AGGREGATE_NONE => graph_item_aggr_fnc2str(AGGREGATE_NONE),
+										AGGREGATE_MIN => graph_item_aggr_fnc2str(AGGREGATE_MIN),
+										AGGREGATE_MAX => graph_item_aggr_fnc2str(AGGREGATE_MAX),
+										AGGREGATE_AVG => graph_item_aggr_fnc2str(AGGREGATE_AVG),
+										AGGREGATE_COUNT => graph_item_aggr_fnc2str(AGGREGATE_COUNT),
+										AGGREGATE_SUM => graph_item_aggr_fnc2str(AGGREGATE_SUM),
+										AGGREGATE_FIRST => graph_item_aggr_fnc2str(AGGREGATE_FIRST),
+										AGGREGATE_LAST => graph_item_aggr_fnc2str(AGGREGATE_LAST)
 									]))
 									->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
 							)
@@ -1141,7 +1315,7 @@ class CWidgetHelper {
 									$field_name.'['.$row_num.'][aggregate_interval]',
 									$value['aggregate_interval']
 								))
-									->setEnabled($value['aggregate_function'] != GRAPH_AGGREGATE_NONE)
+									->setEnabled($value['aggregate_function'] != AGGREGATE_NONE)
 									->setAttribute('placeholder', GRAPH_AGGREGATE_DEFAULT_INTERVAL)
 									->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
 							)
@@ -1152,7 +1326,7 @@ class CWidgetHelper {
 								)
 									->addValue(_('Each item'), GRAPH_AGGREGATE_BY_ITEM)
 									->addValue(_('Data set'), GRAPH_AGGREGATE_BY_DATASET)
-									->setEnabled($value['aggregate_function'] != GRAPH_AGGREGATE_NONE)
+									->setEnabled($value['aggregate_function'] != AGGREGATE_NONE)
 									->setModern(true)
 							)
 					))
@@ -1258,7 +1432,7 @@ class CWidgetHelper {
 
 			'function changeDataSetAggregateFunction(obj) {'.
 				'var row_num = obj.id.replace("ds_", "").replace("_aggregate_function", "");'.
-				'var no_aggregation = (jQuery(obj).val() == '.GRAPH_AGGREGATE_NONE.');'.
+				'var no_aggregation = (jQuery(obj).val() == '.AGGREGATE_NONE.');'.
 				'jQuery("#ds_" + row_num + "_aggregate_interval").prop("disabled", no_aggregation);'.
 				'jQuery("#ds_" + row_num + "_aggregate_grouping_0").prop("disabled", no_aggregation);'.
 				'jQuery("#ds_" + row_num + "_aggregate_grouping_1").prop("disabled", no_aggregation);'.
@@ -1269,7 +1443,7 @@ class CWidgetHelper {
 				'.dynamicRows({'.
 					'template: "#dataset-row",'.
 					'beforeRow: ".'.ZBX_STYLE_LIST_ACCORDION_FOOT.'",'.
-					'remove: ".'.ZBX_STYLE_REMOVE_BTN.'",'.
+					'remove: ".'.ZBX_STYLE_BTN_REMOVE.'",'.
 					'add: "#dataset-add",'.
 					'row: ".'.ZBX_STYLE_LIST_ACCORDION_ITEM.'",'.
 					'dataCallback: function(data) {'.
@@ -1289,7 +1463,7 @@ class CWidgetHelper {
 						jQuery("#widget-dialogue-form")[0].scrollHeight - container.height()
 					));'.
 
-					'jQuery(".input-color-picker input").colorpicker({onUpdate: function(color) {'.
+					'jQuery(".'.ZBX_STYLE_COLOR_PICKER.' input").colorpicker({onUpdate: function(color) {'.
 						'var ds = jQuery(this).closest(".'.ZBX_STYLE_LIST_ACCORDION_ITEM.'");'.
 						'jQuery(".'.ZBX_STYLE_COLOR_PREVIEW_BOX.'", ds).css("background-color", "#"+color);'.
 					'}, appendTo: ".overlay-dialogue-body"});'.
@@ -1351,7 +1525,7 @@ class CWidgetHelper {
 			'});',
 
 			// Initialize color-picker UI elements.
-			'jQuery(".input-color-picker input").colorpicker({onUpdate: function(color){'.
+			'jQuery(".'.ZBX_STYLE_COLOR_PICKER.' input").colorpicker({onUpdate: function(color){'.
 				'var ds = jQuery(this).closest(".'.ZBX_STYLE_LIST_ACCORDION_ITEM.'");'.
 				'jQuery(".'.ZBX_STYLE_COLOR_PREVIEW_BOX.'", ds).css("background-color", "#"+color);'.
 			'}, appendTo: ".overlay-dialogue-body"});',
