@@ -34,7 +34,8 @@ function eventSource($source = null) {
 		EVENT_SOURCE_TRIGGERS => _('trigger'),
 		EVENT_SOURCE_DISCOVERY => _('discovery'),
 		EVENT_SOURCE_AUTOREGISTRATION => _('autoregistration'),
-		EVENT_SOURCE_INTERNAL => _x('internal', 'event source')
+		EVENT_SOURCE_INTERNAL => _x('internal', 'event source'),
+		EVENT_SOURCE_SERVICE => _('service')
 	];
 
 	if ($source === null) {
@@ -61,7 +62,8 @@ function eventObject($object = null) {
 		EVENT_OBJECT_DSERVICE => _('discovered service'),
 		EVENT_OBJECT_AUTOREGHOST => _('autoregistered host'),
 		EVENT_OBJECT_ITEM => _('item'),
-		EVENT_OBJECT_LLDRULE => _('low-level discovery rule')
+		EVENT_OBJECT_LLDRULE => _('low-level discovery rule'),
+		EVENT_OBJECT_SERVICE => _('service')
 	];
 
 	if ($object === null) {
@@ -88,7 +90,8 @@ function eventSourceObjects() {
 		['source' => EVENT_SOURCE_AUTOREGISTRATION, 'object' => EVENT_OBJECT_AUTOREGHOST],
 		['source' => EVENT_SOURCE_INTERNAL, 'object' => EVENT_OBJECT_TRIGGER],
 		['source' => EVENT_SOURCE_INTERNAL, 'object' => EVENT_OBJECT_ITEM],
-		['source' => EVENT_SOURCE_INTERNAL, 'object' => EVENT_OBJECT_LLDRULE]
+		['source' => EVENT_SOURCE_INTERNAL, 'object' => EVENT_OBJECT_LLDRULE],
+		['source' => EVENT_SOURCE_SERVICE, 'object' => EVENT_OBJECT_SERVICE]
 	];
 }
 
@@ -183,7 +186,7 @@ function make_event_details(array $event, array $allowed) {
 		])
 		->addRow([
 			_('Severity'),
-			getSeverityCell($event['severity'])
+			CSeverityHelper::makeSeverityCell((int) $event['severity'])
 		])
 		->addRow([
 			_('Time'),
@@ -487,19 +490,25 @@ function orderEventTagsByPriority(array $event_tags, array $priorities) {
  *                                            - 'triggerid' - for triggers.
  * @param int    $list_tag_count             Maximum number of tags to display.
  * @param array  $filter_tags                An array of tag filtering data.
+ * @param ?array $subfilter_tags             Array of selected sub-filter tags. Null when tags are not clickable.
+ * @param array  $subfilter_tags[<tag>]
+ * @param array  $subfilter_tags[<tag>][<value1>]
+ * @param array  $subfilter_tags[<tag>][<value2>]
+ * @param array  $subfilter_tags[<tag>][<value...>]
  * @param string $filter_tags[]['tag']
  * @param int    $filter_tags[]['operator']
  * @param string $filter_tags[]['value']
  * @param int    $tag_name_format            Tag name format. Possible values:
- *                                            - PROBLEMS_TAG_NAME_FULL (default);
- *                                            - PROBLEMS_TAG_NAME_SHORTENED;
- *                                            - PROBLEMS_TAG_NAME_NONE.
+ *                                            - TAG_NAME_FULL (default);
+ *                                            - TAG_NAME_SHORTENED;
+ *                                            - TAG_NAME_NONE.
  * @param string $tag_priority               A list of comma-separated tag names.
  *
  * @return array
  */
 function makeTags(array $list, bool $html = true, string $key = 'eventid', int $list_tag_count = ZBX_TAG_COUNT_DEFAULT,
-		array $filter_tags = [], int $tag_name_format = PROBLEMS_TAG_NAME_FULL, string $tag_priority = ''): array {
+		array $filter_tags = [], ?array $subfilter_tags = null, int $tag_name_format = TAG_NAME_FULL,
+		string $tag_priority = ''): array {
 	$tags = [];
 
 	if ($html) {
@@ -544,9 +553,19 @@ function makeTags(array $list, bool $html = true, string $key = 'eventid', int $
 				$value = getTagString($tag, $tag_name_format);
 
 				if ($value !== '') {
-					$tags[$element[$key]][] = (new CSpan($value))
-						->addClass(ZBX_STYLE_TAG)
-						->setHint(getTagString($tag));
+					if ($subfilter_tags !== null
+							&& !(array_key_exists($tag['tag'], $subfilter_tags)
+								&& array_key_exists($tag['value'], $subfilter_tags[$tag['tag']]))) {
+						$tags[$element[$key]][] = (new CSimpleButton($value))
+							->setAttribute('data-subfilter-tag', ['subfilter_tags['.$tag['tag'].'][]', $tag['value']])
+							->addClass(ZBX_STYLE_BTN_TAG)
+							->setHint(getTagString($tag), '', false);
+					}
+					else {
+						$tags[$element[$key]][] = (new CSpan($value))
+							->addClass(ZBX_STYLE_TAG)
+							->setHint(getTagString($tag));
+					}
 
 					$tags_shown++;
 
@@ -563,16 +582,25 @@ function makeTags(array $list, bool $html = true, string $key = 'eventid', int $
 
 				foreach ($element['tags'] as $tag) {
 					$value = getTagString($tag);
-					$hint_content[$element[$key]][] = (new CSpan($value))
-						->addClass(ZBX_STYLE_TAG)
-						->setHint($value);
+
+					if ($subfilter_tags !== null
+							&& !(array_key_exists($tag['tag'], $subfilter_tags)
+								&& array_key_exists($tag['value'], $subfilter_tags[$tag['tag']]))) {
+						$hint_content[$element[$key]][] = (new CSimpleButton($value))
+							->setAttribute('data-subfilter-tag', ['subfilter_tags['.$tag['tag'].'][]', $tag['value']])
+							->addClass(ZBX_STYLE_BTN_TAG)
+							->setHint(getTagString($tag), '', false);
+					}
+					else {
+						$hint_content[$element[$key]][] = (new CSpan($value))
+							->addClass(ZBX_STYLE_TAG)
+							->setHint($value);
+					}
 				}
 
-				$tags[$element[$key]][] = (new CSpan(
-					(new CButton(null))
-						->addClass(ZBX_STYLE_ICON_WZRD_ACTION)
-						->setHint(new CDiv($hint_content), '', true, 'max-width: 500px')
-				))->addClass(ZBX_STYLE_REL_CONTAINER);
+				$tags[$element[$key]][] = (new CButton(null))
+					->addClass(ZBX_STYLE_ICON_WIZARD_ACTION)
+					->setHint($hint_content, ZBX_STYLE_HINTBOX_WRAP, true);
 			}
 		}
 		else {
@@ -593,16 +621,16 @@ function makeTags(array $list, bool $html = true, string $key = 'eventid', int $
  * @param array  $tag
  * @param string $tag['tag']
  * @param string $tag['value']
- * @param int    $tag_name_format  PROBLEMS_TAG_NAME_*
+ * @param int    $tag_name_format  TAG_NAME_*
  *
  * @return string
  */
-function getTagString(array $tag, $tag_name_format = PROBLEMS_TAG_NAME_FULL) {
+function getTagString(array $tag, $tag_name_format = TAG_NAME_FULL) {
 	switch ($tag_name_format) {
-		case PROBLEMS_TAG_NAME_NONE:
+		case TAG_NAME_NONE:
 			return $tag['value'];
 
-		case PROBLEMS_TAG_NAME_SHORTENED:
+		case TAG_NAME_SHORTENED:
 			return substr($tag['tag'], 0, 3).(($tag['value'] === '') ? '' : ': '.$tag['value']);
 
 		default:
